/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

(function(window) {

function createCheckerZusatzgeschwindigkeit(deltaX, deltaY) {
    return function() {
        if (!this.pfeile.length)
            return 'Zeichne zuerst die Zusatzgeschwindigkeit ein…';
        const lastPfeil = this.pfeile[this.pfeile.length - 1];
        return (lastPfeil.deltaX == deltaX && lastPfeil.deltaY == deltaY);
    }
}

const LEVELS = [
    {
        "task": "<p>Eine Biene flog von rechts ins Bild, drehte eine Runde und flog wieder rechts aus dem Bild. Nachfolgend siehst du eine Stroboskopaufnahme davon.</p><p>Zeichne für alle Zeitintervalle die Durchschnittsgeschwindigkeit ein. Der Pfeil muss (wie üblich) an der Startposition jedes Intervalls <em>in der Mitte der Biene</em> starten.</p>",
        "shadows": [
            { type: 'image', src: 'bienenflug.svg', x: 0, y: 0, width: 100, height: 60 }
        ],
        check() {
            if (this.pfeile.length != 7)
                return 'Aus den acht abgebildeten Positionen folgt, dass es sieben Zeitintervalle gibt. Du musst also genau sieben Pfeile einzeichnen.';
            const target = [
                {"x":95,"y":20,"deltaX":-20,"deltaY":-10},
                {"x":75,"y":10,"deltaX":-30,"deltaY":-5},
                {"x":45,"y":5,"deltaX":-25,"deltaY":10},
                {"x":20,"y":15,"deltaX":-15,"deltaY":15},
                {"x":5,"y":30,"deltaX":20,"deltaY":15},
                {"x":25,"y":45,"deltaX":30,"deltaY":0},
                {"x":55,"y":45,"deltaX":40,"deltaY":0}
            ];
            const eingabe = target.map(({ x, y }) => this.pfeile.reduce((prev, curr) => prev || ((curr.x1 == x && curr.y1 == y) ? curr : null), null));
            if (eingabe.filter(obj => !obj).length)
                return 'Der Geschwindigkeitspfeil für jedes Zeitintervall muss in der Mitte der Bienenposition beim Zeitintervallstart beginnen';
            const fehler = eingabe.reduce((prev, pfeil, index) => prev || ((pfeil.deltaX != target[index].deltaX || pfeil.deltaY != target[index].deltaY) ? ('Der Pfeil des ' + (index + 1) + '. Zeitintervall ist falsch') : false), false);
            return fehler ? fehler : true;
        }
    },
    {
        "task": "<p>Bestimme die Zusatzgeschwindigkeit der Biene, die sie direkt nach erreichen des höchsten Punktes erhält.</p><p>Zeichne diese Zusatzgeschwindigkeit ein. Du darfst Hilfspfeile einzeichnen, der zuletzt eingezeichnete Pfeil muss aber die Zusatzgeschwindigkeit darstellen.</p>",
        "shadows": [
            { type: 'image', src: 'bienenflug.svg', x: 0, y: 0, width: 100, height: 60 }
        ],
        "style": '#layer1 > path {stroke:#888a85} #layer1 > path:last-child {stroke:#c00}',
        "check": createCheckerZusatzgeschwindigkeit(5, 15)
    },
    {
        "task": "<p>Bestimme die Zusatzgeschwindigkeit der Biene beim Wenden ganz links.</p><p>Zeichne diese Zusatzgeschwindigkeit ein. Du darfst Hilfspfeile einzeichnen, der zuletzt eingezeichnete Pfeil muss aber die Zusatzgeschwindigkeit darstellen.</p>",
        "shadows": [
            { type: 'image', src: 'bienenflug.svg', x: 0, y: 0, width: 100, height: 60 }
        ],
        "style": '#layer1 > path {stroke:#888a85} #layer1 > path:last-child {stroke:#c00}',
        "check": createCheckerZusatzgeschwindigkeit(35, 0)
    }
];

window.karokonstruktionTask = {
    levels: LEVELS
};
})(window);
