/**
* Klasse, die ein anklickbares Kreuz im Koordinatensystem repräsentiert
*/
function Kreuzchen(
    nodeLayer,
    template,
    x,
    y,
    onclick
) {
    var nodeA = template.cloneNode(true);
    nodeA.firstChild.setAttribute('transform', 'translate(' + x + ', ' + y + ')');
    nodeA.addEventListener('click', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        onclick(x, y);
    });
    nodeLayer.appendChild(nodeA);
}


/**
* Klasse, die Spiel steuert
*/
function Simulation(
    countZiel       // Wie viele Richtige in Folge sind nötig
) {
    this.pfeile = [];
    this._establishKreuzchen();
}

/**
* Kreuzchen für Punkte erzeugen
* @return undefined
*/
Simulation.prototype._establishKreuzchen = function() {
    this._nodeSelected = document.getElementById('selected');
    this.resetLastXY();
    var nodeLayer = document.getElementById('layer2');
    var template = document.getElementById('template');
    template.parentNode.removeChild(template);
    template.removeAttribute('id');
    var x, y;
    var self = this;
    for (y = 0; y <= 60; y += 5)
        for (x = 0; x <= 100; x += 5)
            new Kreuzchen(nodeLayer, template, x, y, function(x, y) { self.setLastXY(x, y); if (document.activeElement) document.activeElement.blur(); });
}

// Zuletzt angeklickte Koordinaten
Simulation.prototype.lastXY = null;
// Feld aller Pfeil-Instanzen
Simulation.prototype.pfeile = [];

/**
* Zuletzt angeklickten Punkt zurücksetzen
* @return undefined
*/
Simulation.prototype.resetLastXY = function() {
    this.lastXY = null;
    if (this._nodeSelected && this._nodeSelected.parentNode)
        this._nodeSelected.parentNode.removeChild(this._nodeSelected);
}

/**
* Darauf reagieren, wenn Punkt (x|y) angeklickt wurde
* @return undefined
*/
Simulation.prototype.setLastXY = function(
    x,      // x-Koordinate des angeklickten Punktes
    y       // y-Koordinate des angeklickten Punktes
) {
    if (this.lastXY && ((this.lastXY.x != x) || (this.lastXY.y != y)) && !this.pfeile.reduce((prev, pfeil) => prev || (pfeil.x1 == this.lastXY.x && pfeil.y1 == this.lastXY.y && pfeil.x2 == x && pfeil.y2 == y), false)) {
        this.pfeile.push(new Pfeil(this.lastXY.x, this.lastXY.y, x, y));
        this.resetLastXY();
        this._triggerChanged();
    } else {
        this.lastXY = { x: x, y: y };
        var karos = document.getElementById('karos');
        this._nodeSelected.style.transform = 'translate(' + x + 'px, ' + y + 'px)';
        karos.parentNode.insertBefore(this._nodeSelected, karos.nextSibling);
        this._triggerUndoChanged();
    }
}

/**
* Letzten Schritt rückgängig machen
* @return undefined
*/
Simulation.prototype.undo = function() {
    if (this.lastXY)
        this.resetLastXY();
    else if (this.pfeile.length)
        this.pfeile.pop().remove();
    this._triggerChanged();
}

/**
* Prüft, ob es etwas zum Rückgängigmachen gibt
* @return Boolean   Gibt es etwas zum Rückgängigmachen?
*/
Simulation.prototype.canUndo = function() {
    return (this.pfeile.length || this.lastXY) ? true : false;
}

/**
* Event auslösen, dass sich Status des Update-Buttons evtl. geändert hat
* @return undefined
*/
Simulation.prototype._triggerUndoChanged = function() {
    if (typeof this.onundochange == 'function')
        this.onundochange.call(this, this.canUndo());
}

/**
* Event auslösen, dass sich etwas geändert hat (neuer Strahl oder Strahl entfernt)
* @return undefined
*/
Simulation.prototype._triggerChanged = function() {
    if (typeof this.onchange == 'function')
        this.onchange.call(this, this.pfeile);
    this._triggerUndoChanged();
}

// Methode, die aufgerufen werden soll, wenn sich Anzahl der rückgängig machbaren Schritte geändert hat
Simulation.prototype.onundochange = null;
// Methode, die aufgerufen werden soll, wenn sich etwas geändert hat (neuer Strahl oder Strahl entfernt)
Simulation.prototype.onchange = null;

/**
* Hintergrundelemente einfügen
 * @return undefined
*/
Simulation.prototype.embedShadows = function(
    shadows     // Array aus einzubettenden Daten
) {
    const nodeLayer = document.getElementById('layer0');
    nodeLayer.textContent = '';
    if (!(shadows instanceof Array))
        return;
    shadows.forEach(shadow => {
        if (shadow.type == 'image') {
            ScoTools.createSvg('image', nodeLayer, { x: shadow.x + 2, y: shadow.y + 2, width: shadow.width, height: shadow.height })
                .setAttributeNS('http://www.w3.org/1999/xlink', 'xlink:href', shadow.src);
        }
    });
}

/**
* Alle Pfeile löschen und auch vorausgewähltes Kreuzchen zurücksetzen
* @return undefined
*/
Simulation.prototype.reset = function(
    shadows,    // Array aus einzubettenden Daten
    style       // Zusätzliche CSS-Regeln
) {
    document.getElementById('stylesExtra').textContent = style || '';
    this.embedShadows(shadows);
    this.pfeile.map(function(strahl) { strahl.remove(); });
    this.pfeile = [];
    this.resetLastXY();
}
